@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2005 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
@@ -7,6 +7,10 @@
  *
  * Revisions: 
  *
+ *  03/11/03  [rcg]
+ *            - Buffer now always returned null-terminated.
+ *            - Return value now always -1 when we run out of room.
+ *
  *  01/05/01  [rcg]
  *            - Added Qstrlen, as a safe strlen.
  *
@@ -73,7 +77,8 @@
 
 
 
-#  define BUFSIZE 1024
+enum { BUFSIZE = 1024 };
+enum { NO_SIZE = 2147483647 };
 
 #ifndef MIN
 #  define MIN(A,B) ( ( (A) < (B) ) ? (A) : (B) )
@@ -135,7 +140,7 @@ Qvsnprintf ( char *s, size_t n, const char *format, va_list ap )
                                         *     tokens. */
             *f      = NULL;            /* Pointer into frmToken */
     STATES   nState = IN_FORM; 
-    size_t   nSize  = n-1;
+    size_t   nSize  = n-1;             /* Maximum chars we'll handle */
     size_t   width  = 0;               /* If we have a width minimum */
     BOOL     bWidth = FALSE;           /* TRUE if width specified */
     size_t   limit  = 0;               /* If we have a length limiter */
@@ -185,8 +190,10 @@ Qvsnprintf ( char *s, size_t n, const char *format, va_list ap )
                 if ( bWidth || bLimit )
                     if ( MAX ( width,   limit ) >
                          MIN ( BUFSIZE, nSize )
-                       )
+                       ) {
+                        *s = '\0';
                         return -1;
+                }
 
                 *f++ = *p;
                 *f   = '\0';
@@ -208,8 +215,10 @@ Qvsnprintf ( char *s, size_t n, const char *format, va_list ap )
                 if ( bWidth || bLimit )
                     if ( MAX ( width,   limit ) >
                          MIN ( BUFSIZE, nSize )
-                       )
+                       ) {
+                        *s = '\0';
                         return -1;
+                }
 
                 *f++ = *p;
                 *f   = '\0';
@@ -227,8 +236,10 @@ Qvsnprintf ( char *s, size_t n, const char *format, va_list ap )
                 if ( bWidth || bLimit )
                     if ( MAX ( width,   limit ) >
                          MIN ( BUFSIZE, nSize )
-                       )
+                       ) {
+                        *s = '\0';
                         return -1;
+                }
 
                 *f++ = *p;
                 *f   = '\0';
@@ -246,8 +257,10 @@ Qvsnprintf ( char *s, size_t n, const char *format, va_list ap )
                 if ( bWidth || bLimit )
                     if ( MAX ( width,   limit ) >
                          MIN ( BUFSIZE, nSize )
-                       )
+                       ) {
+                        *s = '\0';
                         return -1;
+                }
 
                 /* 
                  * Get the string pointer.  If NULL, or if the maximum length
@@ -264,8 +277,10 @@ Qvsnprintf ( char *s, size_t n, const char *format, va_list ap )
                 /*
                  * Make sure we have room if limit is unspecified.
                  */
-                if ( bLimit == FALSE && s_len >= nSize )
-                    return -1;
+                if ( bLimit == FALSE && s_len >= nSize ) {
+                        *s = '\0';
+                        return -1;
+                }
                 
                 /* 
                  * Handle minimum width
@@ -313,8 +328,10 @@ Qvsnprintf ( char *s, size_t n, const char *format, va_list ap )
                     f += strlen ( f ); /* on some systems sprintf doesn't
                                           return length of added chars */
                 }
-                else
+                else {
+                    *s = '\0';
                     return -1;
+                }
                 break;
             
             case '0':
@@ -364,9 +381,24 @@ Qvsnprintf ( char *s, size_t n, const char *format, va_list ap )
         } /* We're still inside a format */
     } /* for loop */
     
-    if ( nSize ) 
+    /*
+     * If we haven't run out of room, copy last character of
+     * format string (which is usually the terminating null).
+     * If we have run out of room, just make sure the buffer
+     * is null-terminated.
+     */
+    if ( nSize > 0 ) 
         *s++ = *p;
-    return ( (n-1) - nSize );
+    else
+        *s = '\0';
+
+    /*
+     * Return chars put in buffer (or -1 if we ran out of room)
+     */
+    if ( nSize == 0 && *p != '\0' )
+        return -1;
+    else
+        return ( (n-1) - nSize );
 }
 
 
@@ -390,7 +422,7 @@ Qsprintf ( char *s, const char *format, ... )
     va_list  ap;                       /* Pointer into stack to extract
                                         *     parameters */
     va_start ( ap, format );
-    rslt = Qvsnprintf ( s, 2147483647, format, ap );
+    rslt = Qvsnprintf ( s, NO_SIZE, format, ap );
     va_end   ( ap );
     return rslt;
 }
