@@ -3,13 +3,13 @@
 
      Laurence Lundblade <lgl@qualcomm.com>
 
-     Copyright 2000 QUALCOMM Incorporated.  All rights reserved.
+     Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
      The license.txt file specifies the terms for use, modification, and 
      redistribution.
 
      File: mangle.c - Functions called back by mime parser to do mangling
-     Version: 0.2.3, Dec 1997
-     Last Edited: Dec 17 08:20
+     Version: 0.2.4, Jan 2001
+     Last Edited: Jan 29 08:20
    
      These are the call back functions we supply to the MIME parser. They
      implement all the specific handling of parts that reduced MIME down
@@ -56,6 +56,7 @@
 #  define StrStr     strstr
 #endif /* PILOT */
 
+
 static char *textAttributes[] = { "plain", "html", "enriched" };
 static char *charsetAttributes[] = { "us-ascii",   "iso-8859-1", "iso-8859-2",
                                      "iso-8859-3", "iso-8859-4", "iso-8859-5",
@@ -64,6 +65,18 @@ static char *charsetAttributes[] = { "us-ascii",   "iso-8859-1", "iso-8859-2",
                                      "iso-8859-12","iso-8859-13","iso-8859-15",
                                      "iso-8859-15","utf-8",      "cp1252",
                                      "iso_2022-jp" };
+
+#ifdef    _DEBUG
+#  include "popper.h"
+extern POP *global_debug_p;
+#  define DBG_PRM    global_debug_p, POP_DEBUG, HERE
+#  define DEBUGIT(X) pop_log X /* do { pop_log X; } while FALSE */
+#else
+  #define DBG_PRM ""
+  #define DEBUGIT(X)
+#endif /* _DEBUG */
+
+
 /* ----------------------------------------------------------------------
    822 header handler call back
 
@@ -84,11 +97,7 @@ static char *charsetAttributes[] = { "us-ascii",   "iso-8859-1", "iso-8859-2",
    and thus can't omit whole message/ entities! This is probably a bug
    in the core MIME parser and needs to be rethought.
    ---- */
-void headerHandler( state, header , len, isContinuation )
-void         *state;
-char         *header;
-long          len;
-unsigned char isContinuation;
+void headerHandler ( void *state, char *header, long len, unsigned char isContinuation )
 {
   char       *h;
   ManglerStatePtr mS = (ManglerStatePtr)state;
@@ -665,6 +674,10 @@ int FillMangleInfo ( char *commandLine, char bNoContent,
     
 
     nLen = strlen(commandLine);
+
+    DEBUGIT (( DBG_PRM, "...FillMangleInfo; commandline(%d)=%.50s",
+                 nLen, commandLine ));
+
     /* Initially filled with default values */
     infoStruct->m_text = TextPlain;
     infoStruct->m_char_set = us_ascii;
@@ -690,14 +703,20 @@ int FillMangleInfo ( char *commandLine, char bNoContent,
         int i, n = sizeof(textAttributes)/sizeof(textAttributes[0]);
         p += 5;
         if((q=strchr(p,';')) != NULL || (q=strchr(p,')')) != NULL) {
+            DEBUGIT (( DBG_PRM, "...parsing text attribute (%d) \"%.*s\"",
+                       q-p, q-p, p ));
             for(i=0;i<n;i++){
                 if(strncmp(textAttributes[i],p,q-p) == 0){
                     infoStruct->m_text = (TextAttrType) i;
+                    DEBUGIT (( DBG_PRM, "...matched %s",
+                                textAttributes[i] ));
                     break;
                 }
             }
-            if (i == n) 
+            if (i == n) {
                 return -1;
+                DEBUGIT (( DBG_PRM, "...returning -1 (error)" ));
+            }
         }
     }
     if((p = strstr(commandLine,"charset=")) != NULL) {
@@ -705,19 +724,26 @@ int FillMangleInfo ( char *commandLine, char bNoContent,
         char *q;
         p += 8;
         if((q=strchr(p,';')) != NULL || (q=strchr(p,')')) != NULL) {
+            DEBUGIT (( DBG_PRM, "...parsing charset attribute (%d) \"%.*s\"",
+                        q-p, q-p, p ));
             for(i=0;i<n;i++) {
                 if(strncmp(charsetAttributes[i],p,q-p) == 0) {
                     infoStruct->m_char_set = (TextCharSetType) i;
+                    DEBUGIT (( DBG_PRM, "...matched %s",
+                                 charsetAttributes[i] ));
                     break;
                 }
             }
         }
     }
     if((p = strstr(commandLine,"lang=")) != NULL){
+        DEBUGIT (( DBG_PRM, "...ignoring lang= spec" ));
     }
     if((p = strstr(commandLine,"wrap=")) != NULL){
+        DEBUGIT (( DBG_PRM, "...ignoring wrap= spec" ));
     }
     if((p = strstr(commandLine,"headers=")) != NULL){
+        DEBUGIT (( DBG_PRM, "...parsing headers spec" ));
         infoStruct->m_headers
             = (StringVectorType *)malloc(sizeof(StringVectorType));
         if(infoStruct->m_headers) {
