@@ -1,10 +1,32 @@
 /*
- * Copyright (c) 2001 Qualcomm Incorporated.  All rights reserved.
+ * Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
  * Revisions:
  *
+ *     04/11/05  [rcg]
+ *             - Process user and spool config file as user, not as
+ *               root (fix security hole reported by Jens Steube)
+ *             - Added "xtnd_xmit" as a boolean option to allow XTND XMIT
+ *
+ *     12/03/04  [rcg]
+ *             - Patch by David Shrimpton to always cleanup (not
+ *               just in error cases)
+ *
+ *     08/06/02  [rcg]
+ *             - Added ability to set OpenSSL options (SSL_OP_* values).
+ *               Note that unlike other options, if this is set more
+ *               than once, the values are added together.
+ *
+ *     11/16/01 [secabeen]
+ *             - Added 'home-dir-misc' to put .cache and .pop files in
+ *               user home directories
+ *
+ *     07/25/01 [RCG]
+ *             - Restrict clear-text-password=never to APOP.
+ *             - Restrict clear-text-password=tls to QPOP_SSL
+ *
  *     06/01/01 [RCG]
  *             - Added 'uw-kludge' as synonym for 'uw-kluge'.
  *
@@ -81,6 +103,19 @@
 #include <string.h>
 #include <errno.h>
 #include <ctype.h>
+#include <limits.h>
+
+#if HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
+
+#if HAVE_SYS_UNISTD_H
+#  include <sys/unistd.h>
+#endif
+
+#ifdef        QPOP_OPENSSL
+#  include <openssl/ssl.h>
+#endif     /* QPOP_OPENSSL */
 
 #include "popper.h"
 #include "utils.h"
@@ -104,7 +139,7 @@
  *            restiction can be none, user (can't be set in a user-
  *            controlled file), or init (must be set during
  *            initialization).
- *        3.  Add an case to the switch in the 'get_opt_ptr' function.
+ *        3.  Add a case to the switch in the 'get_opt_ptr' function.
  * 
  * Mnemonic options require three additional steps:
  *        1.  Create an enumeration (usually in popper.h) for the
@@ -188,6 +223,8 @@ typedef enum
     kTLS_PRIVKEYFILE,       /*    (no flag)     */
     kTLS_CIPHERLIST,        /*    (no flag)     */
     kTLS_VERSION,           /*    (no flag)     */
+    kTLS_WORKAROUNDS,       /*    (no flag)     */
+    kTLS_OPTIONS,           /*    (no flag)     */
     kDO_TIMING,             /*    (no flag)     */
     kCHECK_OLD_SPOOL_LOC,   /*    (no flag)     */
     kCHECK_HASH_DIR,        /*    (no flag)     */
@@ -198,6 +235,7 @@ typedef enum
     kGROUP_BULLS,           /*    (no flag)     */
     kHASH_SPOOL,            /*    (no flag)     */
     kHOME_DIR_MAIL,         /*    (no flag)     */
+    kHOME_DIR_MISC,         /*    (no flag)     */
     kOLD_STYLE_UID,         /*    (no flag)     */
     kUW_KLUGE,              /*    (no flag)     */
     kKEEP_TEMP_DROP,        /*    (no flag)     */
@@ -217,6 +255,16 @@ typedef enum
     kLOG_FACILITY,          /* -y facility      */
     kLOG_LOGIN,             /*    (no flag)     */
     kMAXBULLS,              /*    (no flag)     */
+    kSASL_LOG_LOGIN,        /*    (no flag)     */
+    kSASL_MIN_SSF,          /*    (no flag)     */
+    kSASL_MAX_SSF,          /*    (no flag)     */
+    kSASL_NO_PLAINTEXT,     /*    (no flag)     */
+    kSASL_NO_ACTIVE,        /*    (no flag)     */
+    kSASL_NO_DICTIONARY,    /*    (no flag)     */
+    kSASL_FORWARD_SEC,      /*    (no flag)     */
+    kSASL_NO_ANONYMOUS,     /*    (no flag)     */
+    kXTND_XMIT,             /* -x               */
+    kPAM_SESSION,           /*    (no flag)     */
 
 
     LAST_OPT_VAL
@@ -253,6 +301,7 @@ static config_table options [] =
     { "group-server-mode"            , CfgStr  , CfgResUser, kGRP_SERV_MODE      },
     { "hash-spool"                   , CfgInt  , CfgResUser, kHASH_SPOOL         },
     { "home-dir-mail"                , CfgStr  , CfgResUser, kHOME_DIR_MAIL      },
+    { "home-dir-misc"                , CfgBool , CfgResUser, kHOME_DIR_MISC      },
     { "keep-temp-drop"               , CfgBool , CfgResUser, kKEEP_TEMP_DROP     },
     { "kerberos"                     , CfgBool , CfgResUser, kKERBEROS           },
     { "kerberos-service"             , CfgStr  , CfgResUser, kKERBSERVICE        },
@@ -281,6 +330,8 @@ static config_table options [] =
     { "tls-server-cert-file"         , CfgStr  , CfgResInit, kTLS_SERVCERTFILE   },
     { "tls-support"                  , CfgMnem , CfgResInit, kTLS_SUPPORT        },
     { "tls-version"                  , CfgMnem , CfgResInit, kTLS_VERSION        },
+    { "tls-workarounds"              , CfgBool , CfgResUser, kTLS_WORKAROUNDS    },
+    { "tls-options"                  , CfgInt  , CfgResUser, kTLS_OPTIONS        },
     { "tracefile"                    , CfgStr  , CfgResNone, kTRACENAME          },
     { "trim-domain"                  , CfgBool , CfgResUser, kTRIMDOMAIN         },
     { "update-on-abort"              , CfgBool , CfgResNone, kUPDATE_ON_ABORT    },
@@ -288,6 +339,8 @@ static config_table options [] =
     { "user-options"                 , CfgBool , CfgResUser, kUSEROPTS           },
     { "UW-kluge"                     , CfgBool , CfgResNone, kUW_KLUGE           },
     { "UW-kludge"                    , CfgBool , CfgResNone, kUW_KLUGE           },
+    { "xtnd-xmit"                    , CfgBool , CfgResUser, kXTND_XMIT          },
+    { "pam-session"                  , CfgMnem , CfgResUser, kPAM_SESSION        },
 
     { NULL                           , CfgBad  , CfgResNone, LAST_OPT_VAL        }
 };
@@ -321,9 +374,16 @@ typedef struct {
 static mnemonic_map mnem_map_clear_text_pwd [] =
 {
     { "default"         , ClearTextDefault      },
+
+#if defined(APOP) || defined(SCRAM) || defined(CYRUS_SASL)
     { "never"           , ClearTextNever        },
+#endif /* APOP or SCRAM or CYRUS_SASL */
+
+#ifdef    QPOP_SSL
     { "TLS"             , ClearTextTLS          },
     { "SSL"             , ClearTextTLS          },
+#endif /* QPOP_SSL */
+
     { "always"          , ClearTextAlways       },
     { "local"           , ClearTextLocal        },
 
@@ -381,6 +441,16 @@ static mnemonic_map mnem_map_log_facility [] =
 
     { NULL              , PopLogMail            }
 };
+
+/* ...pam-session */
+static mnemonic_map mnem_map_pam_session [] =
+{
+    { "default"         , PamSessionDefault     }, 
+    { "never"           , PamSessionNever       },
+    { "always"          , PamSessionAlways      }, 
+    { NULL              , PamSessionDefault     }
+};
+
     
 
 /*
@@ -393,6 +463,7 @@ static _config_mnemonic_table cfg_mnem_table [] =
     { kTLS_VERSION      , mnem_map_tls_vers       },
     { kCHUNKY_WRITES    , mnem_map_chunky_writes  },
     { kLOG_FACILITY     , mnem_map_log_facility   },
+    { kPAM_SESSION      , mnem_map_pam_session    },
 
     { -1                , NULL                     }
 };
@@ -416,6 +487,11 @@ typedef union
 extern int no_rev_lookup;
 extern int pop_timeout;
 
+/*
+ * Kluge temporary value for items we map to others
+ */
+static int gTempVal = 0;
+
 
 /*
  * Prototypes (static to permit auto-inlining by optimizing compilers)
@@ -459,15 +535,15 @@ tilt ( POP              *p,
 
         case kER_LINE_LEN:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
-                      "Config file %s line %d too long",
-                      cfile, line_no );
+                      "Config file %s line %lu too long",
+                      cfile, (u_long)line_no );
             break;
 
         case kER_EXP_SET:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "Expected \"set\" or \"reset\", found \"%.*s\" "
-                      "at line %d of config file %s",
-                      MIN ( 128, len), token, line_no, cfile );
+                      "at line %lu of config file %s",
+                      MIN ( 128, len), token, (u_long)line_no, cfile );
             break;
 
         case kER_UNKN_OPT:
@@ -491,9 +567,9 @@ tilt ( POP              *p,
                 buf [ strlen ( buf ) - 2 ] = '\0';
             }
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
-                      "Unrecognized option; scanning \"%.*s\" at line %d of "
+                      "Unrecognized option; scanning \"%.*s\" at line %lu of "
                       "config file %s; valid options are: %s",
-                      MIN ( 128, len), token, line_no, cfile, buf );
+                      MIN ( 128, len), token, (u_long)line_no, cfile, buf );
             } /* local block */
             break;
 
@@ -513,10 +589,10 @@ tilt ( POP              *p,
 
             if ( cmp->cfg_value == -1 )
             {
-                pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
-                          "Option \"%s\" does not use mnemonics (line %d "
+                pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are in WHENCE */
+                          "Option \"%s\" does not use mnemonics (line %lu "
                           "of config file %s)",
-                          ctp->name, line_no, cfile );
+                          ctp->name, (u_long)line_no, cfile );
                 break;
             }
 
@@ -537,63 +613,63 @@ tilt ( POP              *p,
             }
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "Unrecognized \"%s\" mnemonic; scanning \"%.*s\" at "
-                      "line %d of config file %s; valid mnemonics are: %s",
-                      ctp->name, MIN ( 128, len), token, line_no, cfile, buf );
+                      "line %lu of config file %s; valid mnemonics are: %s",
+                      ctp->name, MIN ( 128, len), token, (u_long)line_no, cfile, buf );
             } /* local block */
             break;
 
         case kER_RESTR:
             pop_log ( p, POP_DEBUG, fn, ln, /* fn and ln are part of WHENCE */
-                      "Attempt to use restricted option \"%s\" at line %d of "
+                      "Attempt to use restricted option \"%s\" at line %lu of "
                       "user config file %s",
-                      ctp->name, line_no, cfile );
+                      ctp->name, (u_long)line_no, cfile );
             break;
 
         case kER_NOT_BOOL:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "%s is not a boolean option; scanning \"%.*s\" at "
-                      "line %d of config file %s",
-                      ctp->name, MIN ( 128, len), token, line_no, cfile );
+                      "line %lu of config file %s",
+                      ctp->name, MIN ( 128, len), token, (u_long)line_no, cfile );
             break;
 
         case kER_BAD_VAL:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "Bad value for the %s option; scanning \"%.*s\" at "
-                      "line %d of config file %s",
-                      ctp->name, MIN ( 128, len), token, line_no, cfile );
+                      "line %lu of config file %s",
+                      ctp->name, MIN ( 128, len), token, (u_long)line_no, cfile );
             break;
 
         case kER_EXP_BOOL:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "Boolean value expected for the %s option; "
-                      "scanning \"%.*s\" at line %d of config file %s",
-                      ctp->name, MIN ( 128, len), token, line_no, cfile );
+                      "scanning \"%.*s\" at line %lu of config file %s",
+                      ctp->name, MIN ( 128, len), token, (u_long)line_no, cfile );
             break;
 
         case kER_EXP_INT:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "Integer value expected for the %s option; "
-                      "scanning \"%.*s\" at line %d of config file %s",
-                      ctp->name, MIN ( 128, len), token, line_no, cfile );
+                      "scanning \"%.*s\" at line %lu of config file %s",
+                      ctp->name, MIN ( 128, len), token, (u_long)line_no, cfile );
             break;
 
         case kER_EXP_STR:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "String value expected for the %s option; "
-                      "scanning \"%.*s\" at line %d of config file %s",
-                      ctp->name, MIN ( 128, len), token, line_no, cfile );
+                      "scanning \"%.*s\" at line %lu of config file %s",
+                      ctp->name, MIN ( 128, len), token, (u_long)line_no, cfile );
             break;
 
         case kER_EXP_EQL:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "\"=\" expected; found \"%.*s\" at "
-                      "line %d of config file %s",
-                      MIN ( 128, len), token, line_no, cfile );
+                      "line %lu of config file %s",
+                      MIN ( 128, len), token, (u_long)line_no, cfile );
             break;
 
         case kER_RECURS:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
-                      "Config file nesting exceeds %d; will not process "
+                      "Config file nesting exceeds %u; will not process "
                       "config file %s",
                       MAX_RECURSION, cfile );
             break;
@@ -601,30 +677,30 @@ tilt ( POP              *p,
         case kER_EXP_EOL:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "Expected comment or end of line; found \"%.*s\" at "
-                      "line %d of config file %s",
-                      MIN ( 128, len), token, line_no, cfile );
+                      "line %lu of config file %s",
+                      MIN ( 128, len), token, (u_long)line_no, cfile );
             break;
 
         case kER_CONNECTED:
             pop_log ( p, POP_DEBUG, fn, ln, /* fn and ln are part of WHENCE */
                       "The \"%s\" option cannot be set after a client has "
-                      "connected (line %d of config file %s)",
-                      ctp->name, line_no, cfile );
+                      "connected (line %lu of config file %s)",
+                      ctp->name, (u_long)line_no, cfile );
             break;
 
         case kER_NOT_COMPILED_WITH:
             pop_log ( p, POP_DEBUG, fn, ln, /* fn and ln are part of WHENCE */
                       "The \"%s\" option cannot be used because a required "
                       "compile-time option was not set.  See the Administrator's "
-                      "Guide for more information (line %d of config file %s)",
-                      ctp->name, line_no, cfile );
+                      "Guide for more information (line %lu of config file %s)",
+                      ctp->name, (u_long)line_no, cfile );
             break;
 
         case kER_ALREADY_SET:
             pop_log ( p, POP_DEBUG, fn, ln, /* fn and ln are part of WHENCE */
                       "The \"%s\" option has already been set and cannot be set "
-                      "again (line %d of config file %s)",
-                      ctp->name, line_no, cfile );
+                      "again (line %lu of config file %s)",
+                      ctp->name, (u_long)line_no, cfile );
             break;
 
         case kER_HANDLED: /* required error message already issued */
@@ -635,22 +711,22 @@ tilt ( POP              *p,
             int save_err = errno;
             pop_log ( p, POP_DEBUG, fn, ln, /* fn and ln are part of WHENCE */
                       "System error handling the \"%s\" option at line "
-                      "%d of config file %s: %s (%d)",
-                      ctp->name, line_no, cfile, strerror(save_err), save_err );
+                      "%lu of config file %s: %s (%u)",
+                      ctp->name, (u_long)line_no, cfile, strerror(save_err), save_err );
             }
 
         case kER_INTERNAL:
             pop_log ( p, POP_DEBUG, fn, ln, /* fn and ln are part of WHENCE */
                       "Internal error handling the \"%s\" option at line "
-                      "%d of config file %s",
-                      ctp->name, line_no, cfile );
+                      "%lu of config file %s",
+                      ctp->name, (u_long)line_no, cfile );
             break;
 
         default:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "Unknown error; scanning \"%.*s\" at "
-                      "line %d of config file %s",
-                      MIN ( 128, len), token, line_no, cfile );
+                      "line %lu of config file %s",
+                      MIN ( 128, len), token, (u_long)line_no, cfile );
             break;
     } /* switch ( er ) */
 }
@@ -816,6 +892,11 @@ get_opt_ptr ( POP *p, opt_val_type item )
 #define R__PTR(x) ret_val.pPtr=(x);break
 #define R__MNM(x) ret_val.pInt=((int*)(x));break
 
+    /*
+     * First reset the temp global, in case we return a pointer to it.
+     */
+    gTempVal = 0;
+
     switch ( item )
     {
         case kBULLDIR:             R__PTR ( &p->bulldir );
@@ -853,6 +934,8 @@ get_opt_ptr ( POP *p, opt_val_type item )
         case kTLS_PRIVKEYFILE:     R__PTR ( &p->tls_private_key_file );
         case kTLS_CIPHERLIST:      R__PTR ( &p->tls_cipher_list );
         case kTLS_VERSION:         R__MNM ( &p->tls_version );
+        case kTLS_WORKAROUNDS:     R__INT ( &gTempVal );
+        case kTLS_OPTIONS:         R__INT ( &gTempVal );
         case kDO_TIMING:           R__BOO ( &p->bDo_timing );
         case kCHECK_OLD_SPOOL_LOC: R__BOO ( &p->bCheck_old_spool_loc );
         case kCHECK_HASH_DIR:      R__BOO ( &p->bCheck_hash_dir );
@@ -863,6 +946,7 @@ get_opt_ptr ( POP *p, opt_val_type item )
         case kGROUP_BULLS:         R__BOO ( &p->bGroup_bulls );
         case kHASH_SPOOL:          R__INT ( &p->hash_spool );
         case kHOME_DIR_MAIL:       R__PTR ( &p->pHome_dir_mail );
+        case kHOME_DIR_MISC:       R__BOO ( &p->bHome_dir_misc );
         case kOLD_STYLE_UID:       R__BOO ( &p->bOld_style_uid );
         case kUW_KLUGE:            R__BOO ( &p->bUW_kluge );
         case kKEEP_TEMP_DROP:      R__BOO ( &p->bKeep_temp_drop );
@@ -882,6 +966,10 @@ get_opt_ptr ( POP *p, opt_val_type item )
         case kLOG_FACILITY:        R__MNM ( &p->log_facility );
         case kLOG_LOGIN:           R__PTR ( &p->pLog_login );
         case kMAXBULLS:            R__INT ( &p->nMaxBulls );
+        case kXTND_XMIT:           R__BOO ( &p->bXtnd_Xmit );
+#if 0 /* can't use this until we have a way to call pam_close_session with privs */
+        case kPAM_SESSION:         R__MNM ( &p->call_pam_session );
+#endif
 
         default:                   R__PTR ( NULL );
     } /* switch ( item ) */
@@ -984,8 +1072,8 @@ show_result ( POP *p, config_table *opt, int iVal, const char *sVal, WHENCE )
 
         case CfgInt:
             if ( p->debug )
-                pop_log ( p, POP_DEBUG, fn, ln, "Set %s to %d",
-                          opt->name, iVal );
+                pop_log ( p, POP_DEBUG, fn, ln, "Set %s to %d (%#0x)",
+                          opt->name, iVal, iVal );
             break;
 
         case CfgMnem:
@@ -1040,7 +1128,16 @@ handle_value ( POP              *p,
      */
     if ( isdigit ( (int) *pval ) )
     {
-        iVal     = atol ( pval );
+        char *EndPtr = NULL;
+        iVal     = strtol ( pval, &EndPtr, 0 );
+        if ( iVal == LONG_MAX || iVal == LONG_MIN 
+                              || ( iVal == 0 && errno == EINVAL ) )
+        {
+            pop_log ( p, POP_PRIORITY, HERE,
+                      "Invalid value for %s: %.*s",
+                      opt_ent->name, plen, pval );
+            return kER_BAD_VAL;
+        }
         val_type = CfgInt;
     }
     else
@@ -1121,14 +1218,18 @@ handle_value ( POP              *p,
         case kTLS_PASSPHRASE:
             return kER_NOT_COMPILED_WITH;
 #endif /* not QPOP_SSLPLUS */
-            break;
 
 #ifndef       QPOP_OPENSSL
         case kTLS_SERVCERTFILE:
         case kTLS_PRIVKEYFILE:
+        case kTLS_OPTIONS:
             return kER_NOT_COMPILED_WITH;
 #endif /* not QPOP_OPENSSL */
-            break;
+
+#if       ! ( defined ( QPOP_OPENSSL ) && defined ( SSL_OP_ALL ) )
+        case kTLS_WORKAROUNDS:
+            return kER_NOT_COMPILED_WITH;
+#endif /* ! ( defined ( QPOP_OPENSSL ) && defined ( SSL_OP_ALL ) ) */
 
         default:
             break;
@@ -1245,6 +1346,22 @@ handle_value ( POP              *p,
 #endif /* SYSLOG42 */
             break;
 
+#ifdef        QPOP_OPENSSL
+        case kTLS_WORKAROUNDS: /* Set tls_options to SSL_OP_ALL */
+            if ( gTempVal )
+            {
+                p->tls_options = SSL_OP_ALL;
+                DEBUG_LOG1 ( p, "set tls-options to %#0x", p->tls_options );
+            }
+            break;
+
+        case kTLS_OPTIONS: /* Add current value to options */
+            p->tls_options += gTempVal;
+            DEBUG_LOG2 ( p, "added %#0x to tls-options (now %#0x)",
+                         gTempVal, p->tls_options );
+            break;
+#endif     /* QPOP_OPENSSL */
+
         default:
             break;
     } /* switch ( opt_ent->value ) */
@@ -1311,7 +1428,7 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
          * Skip leading white space.
          */
         skip_white_space ( &ptok, &line_len );
-        DEBUG_LOG3 ( p, "...read line %d (%lu): %.256s", 
+        DEBUG_LOG3 ( p, "...read line %lu (%lu): %.256s", 
                     line_no, line_len, ptok );
         
         /*
@@ -1450,14 +1567,14 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
     rslt = POP_SUCCESS;
 
 Exit:
-    if ( rslt != POP_SUCCESS )
-    { /* cleanup */
-        if ( config != NULL )
-        {
-            fclose ( config );
-            config = NULL;
-        }
-    } /* cleanup */
+    /*
+     * cleanup
+     */
+    if ( config != NULL )
+    {
+        fclose ( config );
+        config = NULL;
+    }
 
     DEBUG_LOG2 ( p, "Finished processing config file '%.256s'; rslt=%d",
                  config_file, rslt );
@@ -1473,54 +1590,95 @@ Exit:
 void
 check_config_files ( POP *p, struct passwd *pwp )
 {
-    int         rslt;
-    char        buf [ 256 ];
+    int         rslt_usr        = -1;
+    int         rslt_spl        = -1;
+    char        buf_usr [ 256 ] = "";
+    char        buf_spl [ 256 ] = "";
     struct stat stat_buf;
+    UID_T       uid_save        =  0;
+    BOOL        bUser           = FALSE;
+    BOOL        bSpool          = FALSE;
 
 
+    /*
+     * See if we should process a user config file
+     */
     if ( p->bUser_opts ) {
-        rslt = Qsnprintf ( buf, sizeof(buf), "%s/.qpopper-options", pwp->pw_dir );
-        if ( rslt == -1 )
+        rslt_usr = Qsnprintf ( buf_usr, sizeof(buf_usr), 
+                               "%s/.qpopper-options",
+                               pwp->pw_dir );
+        if ( rslt_usr == -1 )
             pop_log ( p, POP_PRIORITY, HERE, 
                       "Unable to build user options file name for user %s",
                       p->user );
         else {
-            rslt = stat ( buf, &stat_buf );
-            if ( rslt == 0 ) {
-                rslt = pop_config ( p, buf, CfgUser );
-                if ( rslt == POP_FAILURE ) {
-                    pop_log ( p, POP_PRIORITY, HERE,
-                              "Unable to process user options file for user %s",
-                              p->user );
-                }
-            }
+            rslt_usr = stat ( buf_usr, &stat_buf );
+            if ( rslt_usr == 0 )
+                bUser = TRUE;
+            else
+                DEBUG_LOG1 ( p, "User options file %s not present",
+                             buf_usr );
         }
-    } /* p->user_opts */
+    } /* p->bUser_opts */
 
+    /*
+     * See if we should process a spool config file
+     */
     if ( p->bSpool_opts ) {
-        rslt = Qsnprintf ( buf, sizeof(buf), "%s/.%s.qpopper-options",
-                           p->pCfg_spool_dir, p->user );
-        if ( rslt == -1 )
+        rslt_spl = Qsnprintf ( buf_spl, sizeof(buf_spl),
+                               "%s/.%s.qpopper-options",
+                               p->pCfg_spool_dir, p->user );
+        if ( rslt_spl == -1 )
             pop_log ( p, POP_PRIORITY, HERE, 
                       "Unable to build spool options file name for user %s",
                       p->user );
         else {
-            rslt = stat ( buf, &stat_buf );
-            if ( rslt == 0 ) {
-                rslt = pop_config ( p, buf, CfgConnected );
-                if ( rslt == POP_FAILURE ) {
-                    pop_log ( p, POP_PRIORITY, HERE,
-                              "Unable to process spool options file for user %s",
-                              p->user );
-                }
-            }
+            rslt_spl = stat ( buf_spl, &stat_buf );
+            if ( rslt_spl == 0 )
+                bSpool = TRUE;
+            else
+                DEBUG_LOG1 ( p, "Spool options file %s not present",
+                             buf_spl );
         }
-    } /* p->spool_opts */
+    } /* p->bSpool_opts */
+
+    /*
+     * If we are to process either, do it as the user, not root
+     */
+    if ( bUser || bSpool ) {
+        uid_save = geteuid();
+        if ( SETEUID ( pwp->pw_uid ) != 0 ) {
+            rslt_usr = rslt_spl = POP_FAILURE; /* SETEUID failed */
+            pop_log ( p, POP_PRIORITY, HERE,
+                      "SETEUID(%i) for user %s failed",
+                      pwp->pw_uid, p->user );
+        } /* SETEUID failed */
+        else { /* we are now the user */
+            if ( bUser )
+                rslt_usr = pop_config ( p, buf_usr, CfgUser );
+            if ( bSpool )
+                rslt_spl = pop_config ( p, buf_spl, CfgUser );
+
+            if ( SETEUID ( uid_save ) != 0 )
+                pop_log ( p, POP_PRIORITY, HERE,
+                          "SETEUID(%i) back failed", uid_save );
+        } /* we are now the user */
+    } /* bUser || bSpool */
+
+    if ( rslt_usr == POP_FAILURE )
+        pop_log ( p, POP_PRIORITY, HERE,
+                  "Unable to process user options file for user %s",
+                  p->user );
+
+    if ( rslt_spl == POP_FAILURE )
+        pop_log ( p, POP_PRIORITY, HERE,
+                  "Unable to process spool options file for user %s",
+                  p->user );
 }
 
 
 /*
- * To be called by pop_init to set option values.
+ * Called by pop_init to set option values.
  */
 pop_result
 set_option ( POP *p, char *opt_name, char *opt_value )
@@ -1530,7 +1688,7 @@ set_option ( POP *p, char *opt_name, char *opt_value )
     pop_result      err        = POP_FAILURE;
 
 
-    DEBUG_LOG4 ( p, "set_option; opt_name: %d '%s'; opt_value: %d '%s'",
+    DEBUG_LOG4 ( p, "set_option; opt_name: %lu '%s'; opt_value: %lu '%s'",
                  strlen(opt_name), opt_name, strlen(opt_value), opt_value );
 
     /*
