@@ -1,6 +1,6 @@
 
 /*
- * Copyright (c) 2001 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2005 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
@@ -465,7 +465,8 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
     if ( *buffer == MMDF_SEP_CHAR ) {
         p->mmdf_separator = (char *)strdup(buffer);
     } else if ( !isfromline(buffer) ) {
-        DEBUG_LOG5 ( p, "newline=%d; isOflow=%d; wasOflow=%d; buffer(%u):  %.256s", 
+        DEBUG_LOG5 ( p, "newline=%d; isOflow=%d; wasOflow=%d; buffer(%lu):  "
+                        "%.256s", 
                      newline, isOflow, wasOflow, strlen(buffer), buffer );
         return pop_msg ( p, POP_FAILURE, HERE,
                          "[SYS/PERM] Unable to process From lines "
@@ -631,22 +632,22 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
                 if ( !uidl_found ) {
                     char    *cp;
 
-                if ( p->bOld_style_uid == FALSE )
-                    MD5Update ( &mdContext, (unsigned char *)frombuf, 
-                                strlen(frombuf) );
-
+                    if ( p->bOld_style_uid == FALSE )
+                        MD5Update ( &mdContext, (unsigned char *)frombuf, 
+                                    strlen(frombuf) );
+    
                     MD5Final ( digest, &mdContext );
                     cp = mp->uidl_str;
                     cp = encode_uid_hash ( cp, digest, p->bOld_style_uid );
                     *cp++ = '\n';
                     *cp   = '\0';
                     
-                    DEBUG_LOG2 ( p, "UID not found; generated UID(%d): %s",
+                    DEBUG_LOG2 ( p, "UID not found; generated UID(%lu): %s",
                                     strlen(mp->uidl_str), mp->uidl_str );
-
+    
                     mp->length   += strlen("X-UIDL: ") + strlen(mp->uidl_str) +1;
                     p->drop_size += strlen("X-UIDL: ") + strlen(mp->uidl_str) +1;
-
+    
                     /* New UIDs do not dirty the mailspool unless
                      * bUpdate_status_hdrs is set.  They are just recalculated
                      * each time Qpopper is run.
@@ -832,7 +833,7 @@ do_drop_copy ( POP *p, int mfd, time_t time_locked )
         if ( !p->mmdf_separator )
             p->mmdf_separator = (char *)strdup(buffer);
     } else if ( !isfromline ( buffer ) ) {
-        DEBUG_LOG5 ( p, "newline=%d; isOflow=%d; wasOflow=%d; buffer(%u):  %.256s", 
+        DEBUG_LOG5 ( p, "newline=%d; isOflow=%d; wasOflow=%d; buffer(%lu):  %.256s", 
                      newline, isOflow, wasOflow, strlen(buffer), buffer );
         return pop_msg ( p, POP_FAILURE, HERE,
                             "[SYS/PERM] Unable to process From lines "
@@ -1020,7 +1021,7 @@ do_drop_copy ( POP *p, int mfd, time_t time_locked )
                     cp = encode_uid_hash ( cp, digest, p->bOld_style_uid );
                     *cp++ = '\n';
                     *cp   = '\0';
-                    DEBUG_LOG3 ( p, "UID not found; generated UID(%d): %s [%d]",
+                    DEBUG_LOG3 ( p, "UID not found; generated UID(%lu): %s [%d]",
                                     strlen(mp->uidl_str), mp->uidl_str, __LINE__ );
 
                     mp->length   += strlen("X-UIDL: ") + strlen(mp->uidl_str) +1;
@@ -1286,8 +1287,9 @@ struct passwd   * pwp;
         } /* while loop */
 
         if ( p->bull_db == NULL ) {
-            pop_log (p, POP_PRIORITY, HERE, "gdbm_open failed: %s (%d)",
-                        sys_errlist[errno], errno );
+            pop_log (p, POP_PRIORITY, HERE, "gdbm_open failed: %s (%d)"
+                                            "; bull_db: %s",
+                        sys_errlist[errno], errno, bull_db );
             if ( p->bulldb_nonfatal == FALSE )
                 return ( pop_msg ( p, POP_FAILURE, HERE, 
                                    "[SYS/TEMP] Unable to open Bulletin "
@@ -1738,7 +1740,7 @@ struct passwd   * pwp;
          */
         p->server_mode = 0;
         DEBUG_LOG3 ( p, "Unable to open maildrop %s: %s (%d)", 
-                     p->drop_name, (errno < sys_nerr) ? sys_errlist[errno] : "", errno );
+                     p->drop_name, STRERROR(errno), errno );
         cache_unlink ( p );
     }
 
@@ -1875,7 +1877,7 @@ mfgets ( char *s, int size, FILE *stream )
     }
     *p = '\0';
 #ifdef _DEBUG
-    DEBUG_LOG4 ( global_debug_p, "#mfgets: isOflow=%d wasOflow=%d s=(%d) %.128s",
+    DEBUG_LOG4 ( global_debug_p, "#mfgets: isOflow=%d wasOflow=%d s=(%lu) %.128s",
                isOflow, wasOflow, strlen(s), s );
 #endif
     return s;
