@@ -6,12 +6,15 @@
 
 
 /*
- * Copyright (c) 2000 Qualcomm Incorporated.  All rights reserved.
+ * Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
  * Modifications:
  *
+ * 10/13/00  [rcg]
+ *          -  Added LGL's changes for TLS.
+ *
  * 03/01/00  [rcg]
  *          -  Now calling msg_ptr to adjust for hidden msgs and check range.
  *
@@ -38,16 +41,18 @@
 void 
 mangle_count ( void *mstate, char *buf, long len )
 {
-  char *p;
+    char *p;
 
-  *(long *)mstate += len;
+    *(long *)mstate += len;
 
-  for ( p = buf; len > 0; p++, len-- )
-    if ( *p == '\n' )
-      (*(long *)mstate)++;
+    for ( p = buf; len > 0; p++, len-- )
+        if ( *p == '\n' )
+            (*(long *)mstate)++;
 
 }
 
+
+
 /* 
  *  list:   List the contents of a POP maildrop
  */
@@ -190,16 +195,17 @@ POP     * p;
             MimeFinish ( mimeParse );
         } /* show_mangled_length */
 
-        fprintf ( p->output, "%u %lu\r\n", 
-                  mp->visible_num,
-                  (show_mangled_length ? mangled_length : mp->length ) );
+        pop_write_fmt ( p, "%u %lu\r\n", 
+                        mp->visible_num,
+                        (show_mangled_length ? mangled_length
+                                             : mp->length ) );
     } /* for loop */
 
     /*  
      * "." signals the end of a multi-line transmission 
      */
-    fprintf ( p->output, ".\r\n" );
-    fflush  ( p->output );
+    POP_WRITE_LIT   ( p, ".\r\n" );
+    pop_write_flush ( p );
 
     return ( POP_SUCCESS );
 }
