@@ -1,12 +1,15 @@
 /*
- * Copyright (c) 2000 Qualcomm Incorporated.  All rights reserved.
+ * Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  */
 
-#include <config.h>
-#include <stdio.h>
+#include "config.h"
+
 #include <sys/types.h>
+#include <stdio.h>
+#include <stdlib.h>
+
 #include "popper.h"
 
 /* 
