@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
  * The file license.txt specifies the terms for use, modification, or
  * redistribution.
  *
@@ -21,7 +21,7 @@
  */
 
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <sys/types.h>
 
@@ -34,12 +34,12 @@
 int pop_stat (p)
 POP     *   p;
 {
-    DEBUG_LOG2 ( p, "%d visible message(s) (%d octets).", 
+    DEBUG_LOG2 ( p, "%d visible message(s) (%ld octets).", 
                  p->visible_msg_count - p->msgs_deleted,
                  p->drop_size - p->bytes_deleted );
 
     return ( pop_msg ( p, POP_SUCCESS, HERE,
-                       "%u %u",
+                       "%u %lu",
                        p->visible_msg_count - p->msgs_deleted,
                        p->drop_size - p->bytes_deleted ) );
 }
