@@ -477,7 +477,7 @@ open_trace ( char *tname )
 
     uid_save = geteuid();
     myuid = getuid();
-    if ( seteuid ( myuid ) != 0 )
+    if ( SETEUID ( myuid ) != 0 )
         adios ( HERE, "internal error @ %i", __LINE__ );
     
     trace_file = fopen ( tname, "a+" );
@@ -487,7 +487,7 @@ open_trace ( char *tname )
     BLATHER1 ( "Trace and Debug destination is file \"%s\"",
                tname );
             
-    if ( seteuid ( uid_save ) != 0 )
+    if ( SETEUID ( uid_save ) != 0 )
         adios ( HERE, "internal error @ %i", __LINE__ );
 }
 
