@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (c) 2001 Qualcomm Incorporated.  All rights reserved.
+ * Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  */
@@ -15,6 +15,30 @@
  *
  * Revisions:
  *
+ *  12/04/04 [rg]
+ *           - Applied patch from David Shrimpton to unblock sigalarm
+ *             before alarm signal handler is changed.  This is really
+ *             a kludge but helps when (for unclear reasons) the alarm
+ *             signal in the command loop is masked at some time before
+ *             the alarm is set; hence the alarm never occurs, leaving
+ *             hung pop processes which should have timed out waiting
+ *             for a command from client.  [ added debug message when
+ *             SIGALRM turned out to have been masked before kludge ].
+ *
+ *  07/23/04 [rg]
+ *           - free the pw.pw_name field from getpwnam we copied in
+ *             pop_user.c
+ *             (thanks to Kyle McKay for the patch)
+ *
+ *  04/04/02 [rg]
+ *           - getline() now clears out storage buffer when giving up
+ *             after discarding bytes.  Fixes looping DOS attack seen
+ *             on some systems.
+ *
+ *  06/05/01 [rg]
+ *           - Added pop_noop() function to avoid "NOOP has null
+ *             function" log message.
+ *
  *  06/01/01 [rg]
  *           - Added patch by Carles Xavier Munyoz to fix erroneous
  *             scanning for \n in getline().
@@ -277,9 +301,9 @@ DEBUG_LOG1 ( &p,"(v%s) Intro", VERSION );
     for ( p.CurrentState  = auth1;
           p.CurrentState != halt && p.CurrentState != error;
         ) { /* main state loop */
-#ifdef SETPROCTITLE
+#ifdef    HAVE_SETPROCTITLE
         setproctitle ( "%s@%s [%s]: cmd read", p.user, p.client, p.ipaddr );
-#endif
+#endif /* HAVE_SETPROCTITLE */
         DEBUG_LOG3 ( &p, "Qpopper ready for input from %s at %s [%s]",
                      ( *p.user == '\0' ? "(null)" : p.user ),
                      p.client, p.ipaddr );
@@ -306,7 +330,7 @@ DEBUG_LOG1 ( &p,"(v%s) Intro", VERSION );
             if ( s != NULL )  {
                 if ( s->function != NULL ) {
 
-#ifdef    SETPROCTITLE
+#ifdef    HAVE_SETPROCTITLE
                     int i;
                     char command [ 10 ];
 
@@ -404,7 +428,6 @@ char
 *getline ( char *str, int size, POP *pPOP )
 {
     char *p       = NULL;
-    char *pEnd    = NULL;
     int   nRead   = 0;
     int   nRoom   = 0;
     int   nBufSz  = sizeof ( pPOP->pcInBuf );
@@ -470,7 +493,7 @@ char
         } /* nRoom < nBufSz / 2 */
         else
         if ( nRoom == 0 ) {
-            char  junk [ 1024 ];
+            char  junk [ 1024 ] = "";
             int   len  = 0;
             int   disc = 0;
             int   keep = 0;
@@ -482,8 +505,24 @@ char
                     len = pop_tls_read ( pPOP->tls_context, junk, sizeof(junk) );
                 else
                     len = read ( pPOP->input_fd, junk, sizeof(junk) );
-                if ( len <= 0 )
-                    break;
+                if ( len <= 0 ) {
+                    _DEBUG_LOG3 ( pPOP, "...junk read returned %d; ERRNO: %s (%d)",
+                                  len, STRERROR(errno), errno );
+                    if ( errno == EAGAIN || errno == EINTR )
+                        continue; /* There may still be some data */
+                    else {
+                        /*
+                         * Clear out junk in buffer
+                         */
+                        pPOP->pcInEnd   = pPOP->pcInBuf;
+                        pPOP->pcInStart = pPOP->pcInBuf;
+                        nRoom = nBufSz;
+                        _DEBUG_LOG2 ( pPOP, "...set end=%d nRoom=%d",
+                                      pPOP->pcInEnd - pPOP->pcInBuf, nRoom );
+                        *str = '\0'; /* assume EOF */
+                        break;
+                    }
+                }
                 q = strchr ( junk, '\n' );
                 if ( q == NULL ) {
                     disc += len;
@@ -521,7 +560,8 @@ char
                 } /* found a '\n' */
             } /* loop and discard until we see a '\n' */
 
-            _DEBUG_LOG1 ( pPOP, "getline() returning %d", strlen(str) );
+            _DEBUG_LOG2 ( pPOP, "getline() returning %d ('%c')",
+                          strlen(str), *str );
             return ( str );
         } /* nRoom == 0 */
 
@@ -557,7 +597,14 @@ tgetline ( char *str, int size, POP *p, int timeout )
 {
     int ring();
 
+    sigset_t mymask, prevmask;
 
+
+    sigemptyset ( &mymask );
+    sigaddset ( &mymask, SIGALRM );
+    sigprocmask ( SIG_UNBLOCK, &mymask, &prevmask );
+    if ( DEBUGGING && (sigismember(&prevmask, SIGALRM) ) )
+        DEBUG_LOG0 ( p, "SIGALARM was blocked in previous procmask" );
     (void) signal ( SIGALRM, VOIDSTAR ring );
     alarm ( timeout );
     if ( setjmp ( env ) ) {
@@ -568,8 +615,12 @@ tgetline ( char *str, int size, POP *p, int timeout )
     }
     else
         str = getline ( str, size, p );
+
     alarm  ( 0 );
     signal ( SIGALRM, SIG_DFL );
+    sigprocmask ( SIG_UNBLOCK, &mymask, &prevmask );
+    if ( DEBUGGING && (sigismember(&prevmask, SIGALRM) ) )
+        DEBUG_LOG0 ( p, "SIGALARM was blocked in previous procmask" );
     return ( str );
 }
 
@@ -814,11 +865,20 @@ pop_exit ( p, e )
     if ( p->pw.pw_dir != NULL ) {
         free ( p->pw.pw_dir );
     }
+    if ( p->pw.pw_name != NULL ) {
+        free ( p->pw.pw_name );
+    }
 
     p->CurrentState = error;
     return POP_FAILURE;
 }
 
+int
+pop_noop ( POP *p )
+{
+    return pop_msg ( p, POP_SUCCESS, HERE, "no big woop" );
+}
+
 
 char *
 get_state_name ( state curState )
