@@ -5,7 +5,7 @@
  */
  
 /*
- * Copyright (c) 2001 Qualcomm Incorporated.  All rights reserved.
+ * Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  */
@@ -122,7 +122,7 @@
 #   endif
 #endif /* POPSCO */
 
-#if defined(SCOR5) || defined(AUX) || defined(HPUX)
+#if defined(SCOR5) || defined(AUX) || defined(HPUX) || defined(FREEBSD)
 #  define SIGPARAM int foo
 #else
 #  define SIGPARAM 
@@ -192,6 +192,15 @@ typedef enum {
 #  define EPROTO ECONNABORTED
 #endif /* no EPROTO */
 
+/*
+ * Some OSes (notably HPUX) use setresuid() instead of seteuid()
+ */
+#ifdef HAVE_SETRESUID
+#  define SETEUID(E) setresuid(-1,E,-1)
+#else
+#  define SETEUID(E) seteuid(E)
+#endif /* HAVE_SETRESUID */
+
 /*
  * In standalone mode we need to call _exit rather than exit
  */
@@ -439,6 +448,15 @@ typedef enum {                               /* Config file is being processed:
     CfgUser                                  /* - File is under user control    */
 } config_call_type;
 
+/*
+ * Enumeration for when to call pam_*_session functions
+ */
+typedef enum {                               /* Call pam_session_*             */
+    PamSessionDefault = 1,                   /* only if files in home dir used */
+    PamSessionNever,                         /* never                          */
+    PamSessionAlways                         /* always                         */
+} pam_session_type;
+
 
 typedef struct {                                /*  State information for 
                                                     each POP command */
@@ -627,6 +645,7 @@ struct _pop {                                   /*  POP parameter block */
     char            *   tls_passphrase;         /* To decrypt private key  */
     char            *   tls_cipher_list;        /* To restrict ciphers */
     tls_vers_type       tls_version;            /* To restrict TLS/SSL versions */
+    int                 tls_options;            /* Desired SSL_OP_ bit flags */
 
 #ifdef SCRAM
     char                cchal[ MAXLINELEN ];    /* plain text client challenge */ 
@@ -662,6 +681,7 @@ struct _pop {                                   /*  POP parameter block */
     BOOL                bGroup_bulls;           /* Bulletins go to groups */
     int                 hash_spool;             /* Using hashed spools? */
     char               *pHome_dir_mail;         /* Home dir is spool loc */
+    BOOL                bHome_dir_misc;         /* Home dir is .cache/.pop loc */
     BOOL                bOld_style_uid;         /* Generate pre-3.x UIDs */
     BOOL                bUW_kluge;              /* Ignore UW status msgs */
     BOOL                bKeep_temp_drop;        /* Keep temp drop file */
@@ -682,6 +702,10 @@ struct _pop {                                   /*  POP parameter block */
     BOOL               bNo_atomic_open;         /* open() isn't automic. */
     log_facility_type   log_facility;           /* Which log facility to use */
     char               *pLog_login;             /* String to use when logging log-ins */
+    BOOL                bXtnd_Xmit;             /* Allow XTND XMIT */
+#if 0 /* can't use this until we have a way to call pam_close_session with privs */
+    pam_session_type    call_pam_session;       /* When to call pam_session_* */
+#endif
 };
 
 
@@ -743,8 +767,9 @@ int  pop_init         (POP *p, int argcount, char **argmessage);
 int  pop_last         (POP *p);
 int  pop_list         (POP *p);
 int  pop_mdef         (POP *p);
+int  pop_noop         (POP *p);
 int  pop_parse        (POP *p, char *buf);
-int  pop_pass         (POP *p);
+pop_result pop_pass   (POP *p);
 int  pop_quit         (POP *p);
 int  pop_restore      (POP *p);
 int  pop_rpop         (POP *p);
@@ -840,7 +865,8 @@ typedef struct {
 }MDEFRecordType;              /* To maintain MDEF Array defined 
                                  in pop_extend.c. Used by pop_parse
                                  for expansion. */
-#define MAX_MDEF_INDEX 10
+enum { MAX_MDEF_INDEX = 10 };
+enum { MAX_MDEF_NAME  = 100 };
 
 
 MsgInfoList *msg_ptr ( POP *p, int msg_num );
